<?php

    class Write_File_Action extends Reactor_Action
    {
        protected $tpl = 'write_file.php';
        protected $name = 'Write a file to disk';
		
		public static $registration = array(
			ACTION_NAME => 'Write file to disk',
			ACTION_CATEGORY => 'Local',
			ACTION_DESCRIPTION => 'Writes a file to a local directory with appropriate permissions.',
			ACTION_AUTHOR => 'Jeremy Yunis, Nagios Enterprises, LLC',
			ACTION_VERSION => '1.0',
			ACTION_DATE => '01/22/2013', //Last version update
			ACTION_AUTHLEVEL => 0,  // 0 - everyone | 1 - Admin only
			ACTION_TYPE => 'core', //core | user. core modules can't be deleted
			ACTION_META => array(),	//Misc information	
		); 
		
        protected $fields = array(
            'filename' => array(
                'label' => 'filename',
                'validate' => 'trim|required|callback_check_filename'
            ),
            'file_data' => array(
                'label' => 'file data',
                'validate' => 'trim|required|xss_clean|prep_for_form',
            ),
        );

        public function check_filename($str)
        {
            $banned_files = array(
                '/etc/passwd',
                '/etc/hosts',
                '/etc/resolv.conf',
                '/etc/network/interfaces',
                '/etc/sysconfig/network',
            );

            // check for path traversal (dot slash)
            $exp = '/(\.\/)/';
            if(preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename must be absolute.');
                return FALSE;
            }

            // check for a generally acceptable file
            $exp = '/^\/([^\0]+\/)+([^\0]+)$/';
            if(!preg_match($exp, $str)) {
                $this->ci->form_validation->set_message('check_filename', 'The filename was invalid. Please ensure you have entered a valid, absolute UNIX filename and path.');
                return FALSE;
            }
                 
            // check against banned list
            foreach($banned_files as $file) {
                if($file == $str) {
                    $this->ci->form_validation->set_message('check_filename', 'Writing to the file specified is not allowed.');
                    return FALSE;
                }
            }
            return TRUE;
        }

        public function get_label()
        {
            if($filename = $this->get_param('filename')) {
                return sprintf('Write to local file \'%s\'', $filename);
            }
            return parent::get_label();
        }

        public function run($context)
        {
            $this->set_context($context);
            $params = $this->get_params();
            $file = $this->get_param('filename');
            $data = $this->get_param('file_data');
            $this->log('info', "writing file: ".$file);
            if(!is_dir(dirname($file))) {
                $this->log('info', "creating containing folder: ".dirname($file));
                if(!mkdir(dirname($file), 0755, TRUE)) {
                    $this->log('error', "Could not make containing folder.");
                    $this->set_result(0);
                    return $context;
                }
            }
            if($fh = @fopen($file, 'w+')) {
                if(@fwrite($fh, $data) !== FALSE) {
                    $result = 0;
                } else {
                    $result = 1;
                }
                fclose($fh);
            } else {
                $this->log('error', 'Unable to open file. Check that the file exists, is not a folder, and has the correct permissions.');
                $result = 1;
            }
            $this->set_result($result);
            return $context;
        }
    }
